@@ -1,16 +1,11 @@
 use pyo3::exceptions::PyNotImplementedError;
 use pyo3::prelude::*;
-use pyo3::sync::GILOnceCell;
+use pyo3::sync::PyOnceLock;
 use pyo3::types::{PyDict, PyTuple};
 
 use crate::tools::safe_repr;
 
-// see https://github.com/PyO3/pyo3/issues/4894 - freelist is currently unsound with GIL disabled
-#[cfg_attr(
-    not(Py_GIL_DISABLED),
-    pyclass(module = "pydantic_core._pydantic_core", get_all, frozen, freelist = 100)
-)]
-#[cfg_attr(Py_GIL_DISABLED, pyclass(module = "pydantic_core._pydantic_core", get_all, frozen))]
+#[pyclass(module = "pydantic_core._pydantic_core", get_all, frozen, freelist = 100)]
 #[derive(Debug, Clone)]
 pub struct ArgsKwargs {
     pub(crate) args: Py<PyTuple>,
@@ -49,7 +44,7 @@ impl ArgsKwargs {
     }
 }
 
-static UNDEFINED_CELL: GILOnceCell<Py<PydanticUndefinedType>> = GILOnceCell::new();
+static UNDEFINED_CELL: PyOnceLock<Py<PydanticUndefinedType>> = PyOnceLock::new();
 
 #[pyclass(module = "pydantic_core._pydantic_core", frozen)]
 #[derive(Debug)]
