@@ -1,4 +1,5 @@
 use std::borrow::Cow;
+use std::sync::Arc;
 
 use pyo3::intern;
 use pyo3::prelude::*;
@@ -13,7 +14,7 @@ use super::{BuildSerializer, CombinedSerializer, Extra, TypeSerializer};
 #[derive(Debug)]
 pub struct WithDefaultSerializer {
     default: DefaultType,
-    serializer: Box<CombinedSerializer>,
+    serializer: Arc<CombinedSerializer>,
 }
 
 impl BuildSerializer for WithDefaultSerializer {
@@ -22,15 +23,15 @@ impl BuildSerializer for WithDefaultSerializer {
     fn build(
         schema: &Bound<'_, PyDict>,
         config: Option<&Bound<'_, PyDict>>,
-        definitions: &mut DefinitionsBuilder<CombinedSerializer>,
-    ) -> PyResult<CombinedSerializer> {
+        definitions: &mut DefinitionsBuilder<Arc<CombinedSerializer>>,
+    ) -> PyResult<Arc<CombinedSerializer>> {
         let py = schema.py();
         let default = DefaultType::new(schema)?;
 
         let sub_schema = schema.get_as_req(intern!(py, "schema"))?;
-        let serializer = Box::new(CombinedSerializer::build(&sub_schema, config, definitions)?);
+        let serializer = CombinedSerializer::build(&sub_schema, config, definitions)?;
 
-        Ok(Self { default, serializer }.into())
+        Ok(Arc::new(Self { default, serializer }.into()))
     }
 }
 
@@ -43,7 +44,7 @@ impl TypeSerializer for WithDefaultSerializer {
         include: Option<&Bound<'_, PyAny>>,
         exclude: Option<&Bound<'_, PyAny>>,
         extra: &Extra,
-    ) -> PyResult<PyObject> {
+    ) -> PyResult<Py<PyAny>> {
         self.serializer.to_python(value, include, exclude, extra)
     }
 
@@ -71,14 +72,14 @@ impl TypeSerializer for WithDefaultSerializer {
         self.serializer.retry_with_lax_check()
     }
 
-    fn get_default(&self, py: Python) -> PyResult<Option<PyObject>> {
+    fn get_default(&self, py: Python) -> PyResult<Option<Py<PyAny>>> {
         if let DefaultType::DefaultFactory(_, _takes_data @ true) = self.default {
             // We currently don't compute the default if the default factory takes
             // the data from other fields.
             Ok(None)
         } else {
             self.default.default_value(
-                py, &None, // Won't be used.
+                py, None, // Won't be used.
             )
         }
     }
